Files
rocm-systems/api/hip/hip_stream.cpp
T
foreman 7176a5fe81 P4 to Git Change 1552198 by skudchad@skudchad_rocm on 2018/05/08 18:57:32
SWDEV-145570 - [HIP] - Synchronize the legacy default stream with other blocking streams as per the spec.

	ReviewBoardURL = http://ocltc.amd.com/reviews/r/14796/diff/

Affected files ...

... //depot/stg/opencl/drivers/opencl/api/hip/hip_context.cpp#11 edit
... //depot/stg/opencl/drivers/opencl/api/hip/hip_internal.hpp#10 edit
... //depot/stg/opencl/drivers/opencl/api/hip/hip_stream.cpp#5 edit
2018-05-08 19:04:35 -04:00

137 строки
3.8 KiB
C++

/*
Copyright (c) 2015 - present Advanced Micro Devices, Inc. All rights reserved.
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
#include <hip/hip_runtime.h>
#include "hip_internal.hpp"
thread_local std::unordered_set<amd::HostQueue*> streamSet;
static hipError_t ihipStreamCreateWithFlags(hipStream_t *stream, unsigned int flags) {
amd::Device* device = hip::getCurrentContext()->devices()[0];
amd::HostQueue* queue = new amd::HostQueue(*hip::getCurrentContext(), *device, 0,
amd::CommandQueue::RealTimeDisabled,
amd::CommandQueue::Priority::Normal);
if (!(flags & hipStreamNonBlocking)) {
hip::syncStreams();
streamSet.insert(queue);
}
if (queue == nullptr) {
return hipErrorOutOfMemory;
}
*stream = reinterpret_cast<hipStream_t>(as_cl(queue));
return hipSuccess;
}
hipError_t hipStreamCreateWithFlags(hipStream_t *stream, unsigned int flags) {
HIP_INIT_API(stream, flags);
return ihipStreamCreateWithFlags(stream, flags);
}
hipError_t hipStreamCreate(hipStream_t *stream) {
HIP_INIT_API(stream);
return ihipStreamCreateWithFlags(stream, hipStreamDefault);
}
hipError_t hipStreamGetFlags(hipStream_t stream, unsigned int *flags) {
HIP_INIT_API(stream, flags);
amd::HostQueue* hostQueue = reinterpret_cast<amd::HostQueue*>(stream);
auto it = streamSet.find(hostQueue);
if(flags != nullptr) {
*flags = (it != streamSet.end()) ? hipStreamNonBlocking : hipStreamDefault;
} else {
return hipErrorInvalidValue;
}
return hipSuccess;
}
hipError_t hipStreamSynchronize(hipStream_t stream) {
HIP_INIT_API(stream);
amd::HostQueue* hostQueue;
if (stream == nullptr) {
hostQueue = hip::getNullStream();
} else {
hostQueue = as_amd(reinterpret_cast<cl_command_queue>(stream))->asHostQueue();
}
if (hostQueue == nullptr) {
return hipErrorUnknown;
}
hostQueue->finish();
return hipSuccess;
}
hipError_t hipStreamDestroy(hipStream_t stream) {
HIP_INIT_API(stream);
if (stream == nullptr) {
return hipErrorInvalidResourceHandle;
}
amd::HostQueue* hostQueue = reinterpret_cast<amd::HostQueue*>(stream);
streamSet.erase(hostQueue);
as_amd(reinterpret_cast<cl_command_queue>(stream))->release();
return hipSuccess;
}
hipError_t hipStreamWaitEvent(hipStream_t stream, hipEvent_t event, unsigned int flags) {
HIP_INIT_API(stream, event, flags);
assert(0 && "Unimplemented");
return hipErrorUnknown;
}
hipError_t hipStreamQuery(hipStream_t stream) {
HIP_INIT_API(stream);
assert(0 && "Unimplemented");
return hipErrorUnknown;
}
hipError_t hipStreamAddCallback(hipStream_t stream, hipStreamCallback_t callback, void* userData,
unsigned int flags) {
HIP_INIT_API(stream, callback, userData, flags);
assert(0 && "Unimplemented");
return hipErrorUnknown;
}